import os
import traceback

from flask import request, jsonify
from flask_restx import Resource, Namespace, reqparse

from app.defines import StatesCode
from app.utils.create_geojsonl import verify_geotiff

ns = Namespace('image', description='数据管理 API接口')

files_parser = reqparse.RequestParser(bundle_errors=True)
files_parser.add_argument(name='path', type=str, location='args', required=False, help='数据目录')


@ns.route('/verify_geotiff')
class VerifyGeotiffApi(Resource):
    @ns.doc(id='verify', description='验证数据')
    @ns.expect()
    def get(self):
        """数据验证"""
        path = request.args.get('path')
        if not path:
            return jsonify(code=StatesCode.PARA_ERROR, message='输入路径错误')

        err_data = {}

        for file in os.listdir(path):
            if os.path.splitext(file)[-1] == ".tif":
                try:

                    verify_geotiff(os.path.join(path, file))
                except Exception as err:
                    err_data[file] = str(err)

        return jsonify(code=StatesCode.SUCCESS, message='验证完成', data=err_data)
