from osgeo import gdal, osr


class Coordinate:

    EPSG_3857 = None
    EPSG_4326 = None

    def __init__(self, name: str, code: int):

        sr = osr.SpatialReference()

        sr.ImportFromEPSG(code)

        wkt = sr.ExportToWkt()

        self._name = name
        self._code = code

        self._wkt = wkt

        pass

    def __eq__(self, other):

        return self._name == other.name and self._code == other.code

    def __ne__(self, other):

        return self._name != other.name or self._code != other.code

    @property
    def name(self) -> str:

        return self._name

    @property
    def code(self) -> int:

        return self._code

    @property
    def wkt(self) -> str:

        return self._wkt

    @staticmethod
    def warp(dataset, cs):

        return gdal.AutoCreateWarpedVRT \
            (
                dataset,
                dataset.GetProjection(),
                cs.wkt
            )

    @staticmethod
    def from_wkt(wkt):

        if wkt == "":

            return Coordinate("", 0)
            # return Coordinate("EPSG", 4326)

        sr = osr.SpatialReference()

        sr.ImportFromWkt(wkt)

        if sr.IsProjected():

            name = sr.GetAuthorityName("PROJCS")
            code = sr.GetAuthorityCode("PROJCS")

            # gcs = sr.CloneGeogCS()

            pass

        else:

            name = sr.GetAuthorityName("GEOGCS")
            code = sr.GetAuthorityCode("GEOGCS")

            pass

        return Coordinate(name, int(code))

    pass


def EPSG(code):

    return Coordinate("EPSG", code)


Coordinate.EPSG_3857 = EPSG(3857)
Coordinate.EPSG_4326 = EPSG(4326)

