import numpy      as np
import osgeo.gdal as gdal


class DataType:

    def __init__(self, name, bits, min_max=None):

        self._name = name
        self._bits = bits

        self._range = min_max

        pass

    def __str__(self):

        return self._name

    @property
    def name(self):

        return self._name

    @property
    def bits(self):

        return self._bits

    @property
    def range(self):

        return self._range

    def to_gdal(self):

        return DataType._to_gdal[self._name]

    def to_numpy(self):

        return DataType._to_numpy[self._name]

    @staticmethod
    def from_gdal(value, min_max=None):

        name, bits = DataType._from_gdal.get \
            (
                value,
                ('unsupported type: {}'.format(value), 0)
            )

        return DataType(name, bits, min_max)

    @staticmethod
    def from_numpy(value, min_max=None):

        name, bits = DataType._from_numpy.get \
            (
                value.name,
                ('unsupported type: {}'.format(value.name), 0)
            )

        return DataType(name, bits, min_max)

    _from_gdal = \
        {
            gdal.GDT_Byte:    ('uint8',    8),
            gdal.GDT_UInt16:  ('uint16',  16),
            gdal.GDT_UInt32:  ('uint32',  32),
            gdal.GDT_Int16:   ('int16',   16),
            gdal.GDT_Int32:   ('int32',   32),
            gdal.GDT_Float32: ('float32', 32)
        }

    _from_numpy = \
        {
            'uint8':   ('uint8',    8),
            'uint16':  ('uint16',  16),
            'uint32':  ('uint32',  32),
            'int16':   ('int16',   16),
            'int32':   ('int32',   32),
            'float32': ('float32', 32)
        }

    _to_gdal = \
        {
            'uint8':   gdal.GDT_Byte,
            'uint16':  gdal.GDT_UInt16,
            'uint32':  gdal.GDT_UInt32,
            'int16':   gdal.GDT_Int16,
            'int32':   gdal.GDT_Int32,
            'float32': gdal.GDT_Float32
        }

    _to_numpy = \
        {
            'uint8':   np.uint8,
            'uint16':  np.uint16,
            'uint32':  np.uint32,
            'int16':   np.int16,
            'int32':   np.int32,
            'float32': np.float32
        }

    pass

