from .grid import Grid

import morecantile as mct

import numpy as np

import rasterio.features
import rasterio.transform

from starearth import Bound

from supermercado import super_utils  # 临时使用这个函数



class GridMorecantile(Grid):

    def __init__(self, identifier):

        super().__init__(identifier)

        self._tms = mct.tms.get(identifier)

        pass

    def tiles(self, features, z_min, z_max=None):

        bound = Grid.bound_by(features)

        polygons = [f for f in super_utils.filter_features(features)]

        zooms = [z_min] if z_max is None else [z for z in range(z_min, z_max + 1)]

        tiles = []

        for z in zooms:

            minimum = self.tile_of(bound[0], bound[3], z)
            maximum = self.tile_of(bound[2], bound[1], z)

            tile_range = \
                {
                    "x": { "min": minimum[0], "max": maximum[0] + 1 },
                    "y": { "min": minimum[1], "max": maximum[1] + 1 }
                }

            out_shape = \
                [
                    tile_range["y"]["max"] - tile_range["y"]["min"],
                    tile_range["x"]["max"] - tile_range["x"]["min"]
                ]

            transform = rasterio.transform.from_bounds \
                (
                    bound[0],
                    bound[1],
                    bound[2],
                    bound[3],
                    out_shape[0],
                    out_shape[1]
                )

            shapes = [(geom["geometry"], 255) for geom in polygons]

            burn = rasterio.features.rasterize \
                (
                    shapes      = shapes,
                    out_shape   = out_shape,
                    transform   = transform,
                    all_touched = True
                )

            xys = np.fliplr(np.dstack(np.where(burn))[0])

            xys[:, 0] += tile_range["x"]["min"]
            xys[:, 1] += tile_range["y"]["min"]

            tiles.extend(np.append(xys, np.zeros((xys.shape[0], 1), dtype=np.uint8) + z, axis=1).tolist())

            pass

        return tiles

    def bound_of(self, x, y, z):

        b = self._tms.xy_bounds(mct.Tile(x=x, y=y, z=z))

        return Bound.RH(b.left, b.bottom, b.right, b.top)

    def tile_of(self, longitude, latitude, zoom):

        tile = self._tms.tile(longitude, latitude, zoom)

        return tile.x, tile.y, tile.z

    pass

