import os
import numpy as np

from osgeo import gdal
from starearth.output.output import Output


class OutputBIL(Output):

    def __init__(self, filesystem, accessory, nodata=None):

        super().__init__(filesystem, accessory)

        if nodata is None:

            nodata = -9999

        # bil格式nodata默认为0
        # self.nodata = nodata
        self.nodata = 0

        pass

    def write(self, x, y, z, dict_data, properties=None):

        data   = dict_data['data']
        _type  = dict_data['datatype']
        height = dict_data['height']
        width  = dict_data['width']

        ox = dict_data['ox']
        oy = dict_data['oy']
        oh = dict_data['oh']
        ow = dict_data['ow']

        if self._accessory is None:

            exif = None
            date = properties.get('date')
        else:

            exif, date = self._accessory.structure_exif_dict(properties)

            pass

        filename = self._formatter.format(x, y, z, 'bil')

        tiles_path = self._filesystem.path(date).resolve('tiles')

        if not os.path.exists(tiles_path):
            os.makedirs(tiles_path, exist_ok=True)

        path = self._filesystem.path(date).path('tiles').resolve(filename)

        mem_driver = gdal.GetDriverByName('MEM')
        mem_ds = mem_driver.Create('mem', width, height, 1, _type.to_gdal())
        mem_band1 = mem_ds.GetRasterBand(1)
        mem_band1.Fill(self.nodata)
        mem_band1.SetNoDataValue(self.nodata)
        mem_band1.WriteArray(data, ox, oy)

        tile_data = mem_band1.ReadAsArray()

        # tile_data = data.astype(np.int16)

        with open(path, 'wb') as f:

            f.write(tile_data.astype(np.int16).tobytes())
