import io
import os
import traceback
import numpy as np

from PIL import Image
from starearth.output.output import Output


class OutputJPEG(Output):

    def __init__(self, filesystem, accessory, renderer):

        super().__init__(filesystem, accessory)

        self._renderer = renderer

        pass

    def write(self, x, y, z, dict_data, properties=None):

        data = dict_data['data']
        datatype = dict_data['datatype']
        height = dict_data['height']
        width = dict_data['width']
        ox = dict_data['ox']
        oy = dict_data['oy']
        oh = dict_data['oh']
        ow = dict_data['ow']

        data, mode = self._renderer.process(data)

        if data.shape[2] == 4:

            data = data[:, :, :3]  # 在 JPEG 格式中忽略 Alpha 通道
            mode = "RGB"

            pass

        pixels = np.zeros((height, width, 3), np.uint8)  # 经过 renderer 处理之后，数值类型已固定

        pixels[oy:oy + oh, ox:ox + ow] = data

        image = Image.fromarray(pixels, mode)

        if self._accessory is None:

            exif = None
            date = properties.get('date')
        else:

            exif, date = self._accessory.structure_exif_dict(properties)

            pass

        filename = self._formatter.format(x, y, z, 'jpeg')

        # 临时关闭 combine 优化
        # if ox is not None and oy is not None:
        #
        #     filename = self._formatter.c_format(x, y, z, ox, oy, 'jpeg')

        tiles_path = self._filesystem.path(date).resolve('tiles')
        if not os.path.exists(tiles_path):
            os.makedirs(tiles_path, exist_ok=True)

        path = self._filesystem.path(date).path('tiles').resolve(filename)

        if os.path.exists(path):
            with open(path, 'rb') as fp:
                old_bytes = fp.read()

            old_img = Image.open(io.BytesIO(old_bytes)).convert("RGBA")

            image.paste(old_img, (0, 0), old_img)

        if exif:
            image.save(path, exif=exif)

        else:
            image.save(path)
