import os
import numpy as np
from osgeo import gdal
from starearth.datatype import DataType
from starearth.output.output import Output


class OutputRAW(Output):

    def __init__(self, filesystem, accessory, nodata=None):

        super().__init__(filesystem, accessory)

        self._nodata = nodata

        pass

    def write(self, x, y, z, dict_data, properties=None):

        data = dict_data['data']
        datatype = dict_data['datatype']
        height = dict_data['height']
        width = dict_data['width']
        ox = dict_data['ox']
        oy = dict_data['oy']
        oh = dict_data['oh']
        ow = dict_data['ow']

        if len(data.shape) == 2:

            pixels = np.zeros((height, width), data.dtype)

            pixels[oy:oy + oh, ox:ox + ow] = data

            pass

        else:

            pixels = np.zeros((data.shape[0], height, width), data.dtype)

            pixels[:, oy:oy + oh, ox:ox + ow] = data

            pass

        if self._accessory is None:

            exif = None
            date = properties.get('date')
        else:

            exif, date = self._accessory.structure_exif_dict(properties)

            pass

        filename = self._formatter.format(x, y, z, 'tif')

        tiles_path = self._filesystem.path(date).resolve('tiles')
        if not os.path.exists(tiles_path):
            os.makedirs(tiles_path, exist_ok=True)

        path = self._filesystem.path(date).path('tiles').resolve(filename)

        if len(pixels.shape) == 3:
            im_bands, im_height, im_width = pixels.shape
        else:
            im_bands, (im_height, im_width) = 1, pixels.shape

        driver = gdal.GetDriverByName("GTiff")

        dataset = driver.Create \
            (
                path,
                im_height,
                im_width,
                im_bands,
                DataType.from_numpy(pixels.dtype).to_gdal()
            )

        dataset.WriteArray(pixels)

        dataset.SetMetadata(properties)

        bands = dataset.RasterCount

        for band in range(bands):
            if self._nodata._value[band] != None:
                dataset.GetRasterBand(band + 1).SetNoDataValue(self._nodata._value[band])

        del dataset

        pass
