import numpy as np

from starearth import ColorMap


class RendererGreyscale:

    def __init__(self, datatype, lut=None, nodata=None):

        if lut is None:

            # 如果没有指定 LUT，使用默认（纯黑到纯白）

            lut = ColorMap().data()

            pass

        if nodata is not None:

            # 构造透明通道

            alpha = np.ones(len(lut), np.uint8) * 255

            alpha[nodata] = 0

            # 追加透明通道至 LUT

            lut = np.concatenate((lut, np.expand_dims(alpha, axis=-1)), axis=-1)

            # 模式变为 RGBA

            self._mode = 'RGBA'

            pass

        else:

            self._mode = 'RGB'

            pass

        self._lut = lut

        if datatype.name == "uint8":

            self._value_range = [0, 255]

            pass

        else:

            # 目前默认，将所有非 uint8 类型，使用其最大最小值映射至颜色表两端（拉伸）

            self._value_range = datatype.range

            pass

        self._bias = 0  # 这个值在处理时会被加到像素数据上，使得整体值范围保持在[0, R[1] - R[0]]

        if self._value_range[0] < 0.0:

            self._bias = -self._value_range[0]

            pass

        # 保存一个系数，避免产生重复计算

        self._factor = (len(self._lut) - 1) / (self._value_range[1] - self._value_range[0])

        pass

    def process(self, image):

        indices = np.asarray(image + self._bias, int) * self._factor

        return np.take(self._lut, indices.astype(int), 0), self._mode

    pass

