import numpy as np
from starearth import DataType


class RendererRGB:

    _datatype: DataType

    def __init__(self, datatype: DataType, channels: [int] = None, nodata = None):

        self._datatype = datatype

        if channels is None:

            channels = [1, 2, 3]

            pass

        self._channels = channels
        self._nodata   = nodata

        pass

    def process(self, image):

        data = np.asarray([image[i - 1] for i in self._channels], np.float32)

        # 无论 image 为何种数据类型，输出数据类型固定为 uint8

        if self._datatype.name != 'uint8':

            if self._datatype.range[0] < 0.0 or 255.0 < self._datatype.range[1]:

                # 只有超过 uint8 数值范围的数据需要拉伸

                data = data * 256.0 / (2 ** self._datatype.bits)

                pass

            pass

        data = np.stack(data, -1).astype(np.uint8)

        if self._nodata is None:

            return data, 'RGB'

        alpha = np.zeros((data.shape[0], data.shape[1]), data.dtype)

        alpha[np.all(data != self._nodata, -1)] = 255

        data = np.concatenate((data, np.expand_dims(alpha, axis=-1)), axis=-1)

        return data, 'RGBA'

    pass

