import math
import multiprocessing as mp
import numpy           as np

from PIL         import Image

from .coordinate import EPSG, Coordinate
from .grid       import Grid, GridFactory
from .scene      import Scene



def process_sample(filename, grid, tiles, square, output, cache, date, properties, total_num, epsg, coordinate=None, span=None):

    from .scene import Scene

    scene = Scene()

    scene.load(filename)

    if coordinate:

        scene.project(coordinate)

        pass

    grid = GridFactory.create(grid)

    increment = 1 / total_num

    # slice_each = Slice_Each()
    # for x, y, z in slice_each(tiles, increment):
    for x, y, z in tiles:

        bound = grid.bound_of(x, y, z)

        dict_data = scene.sample(bound, square, square, span)

        output.write(x, y, z, dict_data, properties)

        pass

    pass


def process_combine(filename, tiler, tiles, square, output, cache, date, total_num):
    # 创建单个底板 blank
    blank = Image.fromarray(np.zeros((square, square, 3), np.uint8), 'RGB')

    for x, y, z in tiles:
        # 建立一个新的背景板
        canvas = Image.fromarray(np.zeros((square * 2, square * 2, 3), np.uint8), 'RGB')

        tz = z + 1

        # 计算z级的 x y位置瓦片，由四个 z + 1 的瓦片合并的 tx ty取值
        tx0 = x * 2
        tx1 = tx0 + 1
        ty0 = y * 2
        ty1 = ty0 + 1

        im_00, dict_00 = cache.take(tx0, ty0, tz, blank, date)
        im_10, dict_10 = cache.take(tx1, ty0, tz, blank, date)
        im_01, dict_01 = cache.take(tx0, ty1, tz, blank, date)
        im_11, dict_11 = cache.take(tx1, ty1, tz, blank, date)

        # 将四个z+1级图像，分别放到：左上角，右上角，左下角，右下角
        canvas.paste(im_00, (0 + dict_00['ox'], 0 + dict_00['oy']))
        canvas.paste(im_10, (square + dict_10['ox'], 0 + dict_10['oy']))
        canvas.paste(im_01, (0 + dict_01['ox'], square + dict_01['oy']))
        canvas.paste(im_11, (square + dict_11['ox'], square + dict_11['oy']))

        ## canvas.paste(im_00, (0, 0))
        ## canvas.paste(im_01, (square, 0))
        ## canvas.paste(im_10, (0, square))
        ## canvas.paste(im_11, (square, square))

        image = canvas.resize((square, square), Image.BOX)

        cache.put(x, y, z, image)

        output.write(x, y, z, image)

        # #获取canvas的最小包围盒的宽高

        # width_00, height_00 = im_00.size[0], im_00.size[-1]
        # width_01, height_01 = im_01.size[0], im_01.size[-1]
        # width_10, height_10 = im_10.size[0], im_10.size[-1]
        # width_11, height_11 = im_11.size[0], im_11.size[-1]

        # width_00_10 = width_00 + width_10
        # width_01_11 = width_01 + width_11
        #
        # # canvas_width=width_00_10
        # canvas_width=max( width_00_10, width_01_11)
        #
        # height_00_01 = height_00 + height_01
        # height_10_11 = height_10 + height_11
        #
        # # canvas_height = height_00_01
        # canvas_height = max(height_00_01, height_10_11)
        #
        # canvas_min = Image.fromarray(np.zeros((canvas_width, canvas_height, 3), np.uint8), 'RGB')
        #
        # # 将四个z+1级图像，分别放到：左上角，右上角，左下角，右下角 （右下角位置 需要根据 其他三个的宽高确定）
        # if im_00:
        #     canvas_min.paste(im_00, (0, 0))
        #     if im_10:
        #         canvas_min.paste(im_10, (width_00, 0))
        #     if im_01:
        #         canvas_min.paste(im_01, (0, height_00))
        #     if im_11:
        #         # canvas_min.paste(im_11, (width_00, height_00))
        #         canvas_min.paste(im_11, (max(width_00, width_10), max(height_00, height_01)))
        #
        #     # 保存一个标准大小的图像数据 根据左上角的图像 (还会有黑线 但是先合成在 放入)
        #     canvas.paste(canvas_min, (0 + dict_00['ox'], 0 + dict_00['oy']))
        # else:
        #     if im_01:
        #         canvas_min.paste(im_01, (0, 0))
        #         # canvas_min.paste(im_11, (width_00, height_00))
        #         if im_11:
        #             canvas_min.paste(im_11, (width_10, 0))
        #
        #             # 保存一个标准大小的图像数据 根据左上角的图像 (还会有黑线 但是先合成在 放入)
        #             canvas.paste(canvas_min, (square + dict_01['ox'], 0 + dict_01['oy']))
        #
        # # # 保存一个标准大小的图像数据 根据左上角的图像 (还会有黑线 但是先合成在 放入)
        # # canvas.paste(canvas_min, (0 + dict_00['ox'], 0 + dict_00['oy']))
        #
        # image = canvas.resize((square, square), Image.BOX)
        #
        # cache.put(x, y, z, image)
        #
        # output.write(x, y, z, image)

        # image = canvas_min.resize((math.ceil(canvas_width/2), math.ceil(canvas_height/2)), Image.BOX)
        #
        # # 如果缩小后图像与 标准大小不一致 则需要记录一下该图像 为后续四合一做准备
        # if image.height != square or image.width != square:
        #
        #     cache.put(x, y, z, image)
        #
        #     output.write(x, y, z, image, ox=dict_00['ox'], oy=dict_00['oy'])

        pass

    pass


def cbe(exception):

    print(exception)

    pass


class Slicer:

    _scene: Scene
    _grid:  Grid

    def __init__(self, scene: Scene, square: int, output, grid: Grid):

        self._scene  = scene
        self._square = square
        self._output = output
        self._grid   = grid


    def slice(self, z_min: int, z_max: int, epsg, sheets=None, span=None, coordinate=None):

        assert z_min <= z_max

        # count = mp.cpu_count()
        count = math.ceil(mp.cpu_count() / 2)

        with mp.Manager() as manager:

            cache = None

            if sheets:

                for sheet in sheets:

                    # bound = Grid.bound_by(sheet.features)
                    feature = sheet.features

                    properties = sheet.features[0]['properties']

                    # tiles = self._grid.tiles(bound, z_min, z_max)
                    tiles = self._grid.tiles(feature, z_min, z_max)

                    total_num = len(tiles)

                    # 从最大级别开始，依次降级，多进程切片（拼合）

                    for z in np.flipud(range(z_min, z_max + 1)):

                        self._run_mp(cache, z, process_sample, count, sheet.date, feature, properties, total_num, epsg, coordinate, span)

            else:

                bound = self._scene.bound.value

                properties = {}
                tiles = self._grid.tiles(bound, z_min, z_max)
                total_num = len(tiles)

                for z in np.flipud(range(z_min, z_max + 1)):

                    self._run_mp(cache, z, process_sample, count, 'default', bound, properties, total_num, epsg, coordinate, span)

    def _run_mp(self, cache, z, function, processes, date, feature, properties, total_num, epsg, coordinate, span):

        tiles = self._grid.tiles(feature, z)

        areas = np.array_split(tiles, processes)

        pool = mp.Pool(processes)

        for area in areas:

            if len(area) == 0:

                continue
                pass

            pool.apply_async \
                (
                    function,
                    (
                        self._scene.filename,
                        self._grid.identifier,  # Grid 对象无法被 pickle，所以传标识符进去
                        area,
                        self._square,
                        self._output,
                        cache,
                        date,
                        properties,
                        total_num,
                        epsg,
                        coordinate,
                        span
                    ),
                    error_callback=cbe
                )

            pass

        pool.close()
        pool.join()

        pass

    def _slice_sp(self, z_min, z_max, epsg, sheets=None, span=None, coordinate=None):

        # 单进程切片，仅调试用

        assert z_min <= z_max

        cache = None

        if sheets:

            for sheet in sheets:

                bound = Grid.bound_by(sheet.features)

                for z in np.flipud(range(z_min, z_max + 1)):

                    tiles = self._grid.tiles(bound, z)

                    areas = np.array_split(tiles, 1)

                    for area in areas:

                        process_sample \
                            (
                                self._scene.filename,
                                self._grid.identifier,
                                area,
                                self._square,
                                self._output,
                                cache,
                                sheet.date,
                                None,
                                1,
                                epsg,
                                coordinate,
                                span,
                            )


        else:

            bound = self._scene.bound.value

            for z in np.flipud(range(z_min, z_max + 1)):

                tiles = self._grid.tiles(bound, z)

                areas = np.array_split(tiles, 1)

                for area in areas:

                    process_sample \
                        (
                            self._scene.filename,
                            self._grid.identifier,
                            area,
                            self._square,
                            self._output,
                            cache,
                            'default',
                            None,
                            1,
                            epsg
                        )
