import os
import json
import shutil
from PIL import Image
from starearth.utils.tileset import TileSet
from tilecloud import TileStore, Tile, TileCoord


class StorageARCZYX:
    def __init__(self, tiles_path, min_zoom, max_zoom, merging, output_path, tileset):
        self.tiles_path          = tiles_path
        self.min_zoom            = min_zoom
        self.max_zoom            = max_zoom
        self.merging             = merging
        self.output_path         = output_path
        self.tileset_id          = tileset.get('id', '')
        self.tileset_name        = tileset.get('name', '')
        self.tileset_owner       = tileset.get('owner', '')
        self.tileset_tile_grid   = tileset.get('tile_grid', '')
        self.tileset_attribution = tileset.get('attribution', '')
        self.tileset_description = tileset.get('description', '')

    def storage(self):
        tile_set             = TileSet()
        tile_set.id          = self.tileset_id
        tile_set.minzoom     = self.min_zoom
        tile_set.maxzoom     = self.max_zoom
        tile_set.name        = self.tileset_name
        tile_set.owner       = self.tileset_owner
        tile_set.tile_grid   = self.tileset_tile_grid
        tile_set.attribution = self.tileset_attribution
        tile_set.description = self.tileset_description

        tile_set.bounds = []
        msmt_list = list()
        date_list = []

        # 开始处理切片目录下的瓦片
        # for tile_dir in os.listdir(self.tiles_path):
        #
        #     abs_tiles_path = os.path.join(self.tiles_path, tile_dir)

        abs_tiles_path = self.tiles_path

        for tile_date in os.listdir(abs_tiles_path):

            date_dir_path = os.path.join(abs_tiles_path, tile_date)

            if not os.path.isdir(date_dir_path):
                continue

            meta_json_path = os.path.join(date_dir_path, 'meta.json')
            imagesheet_geojsonl_path = os.path.join(date_dir_path, 'imagesheet.geojsonl')
            tile_list_path = os.path.join(date_dir_path, 'tiles_list.json')

            with open(meta_json_path, 'r') as fb:
                data = json.load(fb)
                bbox = data['bbox']
                date_meta_min_z = int(data['min_z'])
                date_meta_max_z = int(data['max_z'])
                tile_format = data['tileset']['tile_format']

            if tile_set.bounds == []:
                tile_set.bounds = bbox

            else:
                _min_lon = min(tile_set.bounds[0], bbox[0])
                _min_lat = min(tile_set.bounds[1], bbox[1])
                _max_lon = min(tile_set.bounds[2], bbox[2])
                _max_lat = min(tile_set.bounds[3], bbox[3])

                tile_set.bounds = [_min_lon, _min_lat, _max_lon, _max_lat]

                pass
            # 获取瓦片列表
            with open(tile_list_path, 'r') as fb:
                tile_list = json.load(fb)
            # 拼接瓦片源路径和目的路径
            local_tiles_src = "file://{}/tiles/%(z)d_%(y)d_%(x)d.{}".format(date_dir_path                            , tile_format)
            local_tiles_dst = "file://{}/tiles/%(z)d/%(y)d/%(x)d.{}".format(os.path.join(self.output_path, tile_date), tile_format)

            ts_input = TileStore.load(local_tiles_src)
            ts_output = TileStore.load(local_tiles_dst)

            # 使用瓦片列表构造Tile对象
            tiles = [Tile(TileCoord(z, x, y)) for x, y, z in tile_list]

            for tile in tiles:
                tile = ts_input.get_one(tile)

                if tile is None:
                    continue

                if tile in ts_output:
                    if self.merging == 0:
                        # 仅保留原瓦片
                        continue

                    elif self.merging == 1:
                        # 替换原瓦片
                        ts_output.put_one(tile)

                    elif self.merging == 2:
                        # 合并同名瓦片（原瓦片在上）
                        z, x, y = [int(i) for i in str(tile.tilecoord).split('/')]

                        origin_tile_path = local_tiles_dst.replace('file://', '') % {'z': z, 'y': y, 'x': x}
                        new_tile_path    = local_tiles_src.replace('file://', '') % {'z': z, 'y': y, 'x': x}

                        origin_img = Image.open(origin_tile_path)
                        new_img = Image.open(new_tile_path)

                        new_img.paste(origin_img, (0, 0), origin_img)

                        new_img.save(origin_tile_path)

                        pass

                    elif self.merging == 3:
                        # 合并同名瓦片（原瓦片在下）

                        z, x, y = [int(i) for i in str(tile.tilecoord).split('/')]

                        origin_tile_path = local_tiles_dst.replace('file://', '') % {'z': z, 'y': y, 'x': x}
                        new_tile_path    = local_tiles_src.replace('file://', '') % {'z': z, 'y': y, 'x': x}

                        origin_img = Image.open(origin_tile_path)
                        new_img = Image.open(new_tile_path)

                        origin_img.paste(new_img, (0, 0), new_img)

                        origin_img.save(origin_tile_path)

                        pass

                else:
                    # 存储瓦片
                    ts_output.put_one(tile)

            if not os.path.exists(os.path.join(self.output_path, tile_date)):
                os.makedirs(os.path.join(self.output_path, tile_date), exist_ok=True)

            # 合并meta.json
            dst_meta_path = os.path.join(self.output_path, tile_date, 'meta.json')
            if os.path.exists(dst_meta_path):
                with open(dst_meta_path, 'r') as fb:
                    dst_meta_data = json.load(fb)
                    dst_meta_bbox = dst_meta_data['bbox']
                    dst_meta_min_z = int(dst_meta_data['min_z'])
                    dst_meta_max_z = int(dst_meta_data['max_z'])

                dst_meta_min_z = min(date_meta_min_z, dst_meta_min_z)
                dst_meta_max_z = max(date_meta_max_z, dst_meta_max_z)

                left = min(dst_meta_bbox[0], bbox[0])
                bottom = min(dst_meta_bbox[1], bbox[1])
                right = max(dst_meta_bbox[2], bbox[2])
                top = max(dst_meta_bbox[3], bbox[3])

                dst_meta_data['bbox'] = [left, bottom, right, top]

                dst_meta_data['min_z'] = dst_meta_min_z
                dst_meta_data['max_z'] = dst_meta_max_z

                dst_meta_data['geometry']['coordinates'] = [
                    [[left, top], [right, top], [right, bottom], [left, bottom], [left, top]]]

                with open(dst_meta_path, 'w') as fb:
                    json.dump(dst_meta_data, fb)

            else:
                shutil.copy(meta_json_path, os.path.join(self.output_path, tile_date))

            # 合并imagesheet.geojsonl
            dst_imagesheet_path = os.path.join(self.output_path, tile_date, 'imagesheet.geojsonl')
            if os.path.exists(dst_imagesheet_path):

                with open(imagesheet_geojsonl_path, 'r') as fb:
                    new_imagesheet_data = json.load(fb)

                with open(dst_imagesheet_path, 'a+') as fb:
                    fb.write(json.dumps(new_imagesheet_data))
            else:
                shutil.copy(imagesheet_geojsonl_path, os.path.join(self.output_path, tile_date))

            msmtdate = {
                tile_date: {
                    "storage_type": "ARC_ZYX",
                    "path": "ARC_ZYX://{}/{}".format(self.output_path, tile_date)
                }
            }

            if tile_date not in date_list:
                msmt_list.append(msmtdate)
                date_list.append(tile_date)

        # 经纬度中心点
        tile_set.center = [
            (tile_set.bounds[0] + tile_set.bounds[2]) / 2,
            (tile_set.bounds[1] + tile_set.bounds[3]) / 2,
            self.min_zoom
        ]

        tile_set.msmt_info = msmt_list

        tile_set.save(path=self.output_path)

        pass
