import multiprocessing
import os
import json

import numpy
from PIL import Image
from tilecloud import TileStore, Tile, TileCoord


def cbe(exception):
    print(exception)

    pass


class StorageOSMZXY:
    def __init__(self, tiles_path, min_zoom, max_zoom, merging, output_path, tile_format):
        self.tiles_path = tiles_path
        self.min_zoom = min_zoom
        self.max_zoom = max_zoom
        self.merging = merging
        self.output_path = output_path
        self.tile_format = tile_format

    def _run_mp(self, tiles, ts_input, ts_output, local_tiles_dst, local_tiles_src):
        for tile in tiles:
            tile = ts_input.get_one(tile)

            if tile is None:
                continue

            if tile in ts_output.get_all():
                if self.merging == 0:
                    # 仅保留原瓦片
                    continue

                elif self.merging == 1:
                    # 替换原瓦片
                    ts_output.put_one(tile)

                elif self.merging == 2:
                    # 合并同名瓦片（原瓦片在上）

                    z, x, y = [int(i) for i in str(tile.tilecoord).split('/')]

                    origin_tile_path = local_tiles_dst.replace('file://', '') % {'z': z, 'x': x, 'y': y}
                    new_tile_path = local_tiles_src.replace('file://', '') % {'z': z, 'x': x, 'y': y}

                    origin_img = Image.open(origin_tile_path)
                    new_img = Image.open(new_tile_path)

                    new_img.paste(origin_img, (0, 0), origin_img)

                    new_img.save(origin_tile_path)

                    pass

                elif self.merging == 3:
                    # 合并同名瓦片（原瓦片在下）

                    z, x, y = [int(i) for i in str(tile.tilecoord).split('/')]

                    origin_tile_path = local_tiles_dst.replace('file://', '') % {'z': z, 'x': x, 'y': y}
                    new_tile_path = local_tiles_src.replace('file://', '') % {'z': z, 'x': x, 'y': y}

                    origin_img = Image.open(origin_tile_path)
                    new_img = Image.open(new_tile_path)

                    origin_img.paste(new_img, (0, 0), new_img)

                    origin_img.save(origin_tile_path)

                    pass

            else:
                # 存储瓦片
                ts_output.put_one(tile)

    def storage(self):
        # 开始处理切片目录下的瓦片

        date_dir_path = os.path.join(self.tiles_path, 'tiles')
        tile_list_path = os.path.join(self.tiles_path, 'tiles_list.json')

        # 获取瓦片列表
        with open(tile_list_path, 'r') as fb:
            tile_list = json.load(fb)

        # 拼接瓦片源路径和目的路径
        local_tiles_src = "file://{}/%(z)d_%(y)d_%(x)d.{}".format(date_dir_path, self.tile_format)
        local_tiles_dst = "file://{}/%(z)d/%(x)d/%(y)d.{}".format(self.output_path, self.tile_format)

        ts_input = TileStore.load(local_tiles_src)
        ts_output = TileStore.load(local_tiles_dst)

        # 使用瓦片列表构造Tile对象
        tiles = [Tile(TileCoord(z, x, y)) for x, y, z in tile_list]

        datas = numpy.array_split(tiles, self.max_zoom - self.min_zoom)

        # for data in datas:
        #     process = multiprocessing.Process(
        #         target=self._run_mp,
        #         args=(
        #             data,
        #             ts_input,
        #             ts_output,
        #             local_tiles_src,
        #             local_tiles_dst
        #         )
        #     )
        #     print(111)
        #     process.start()
        #     process.join()

        pool = multiprocessing.Pool(self.max_zoom - self.min_zoom)

        for data in datas:
            pool.apply_async \
                    (
                    self._run_mp,
                    (
                        data,
                        ts_input,
                        ts_output,
                        local_tiles_src,
                        local_tiles_dst
                    ),
                    error_callback=cbe
                )

        pool.close()
        pool.join()
