from starearth.task.storage_mongo import TaskStorageMongo
from starearth.task.ptp_package import TaskStoragePTP


class StorageFactory(object):
    """工厂类，选择创建哪种打包方式"""

    def __init__(self):
        self._creators = {
            'StarEarth_PTP':   TaskStoragePTP,
            'StarEarth_Mongo': TaskStorageMongo,
        }

    _instance = None

    @staticmethod
    def _singleton():
        if StorageFactory._instance is None:
            StorageFactory._instance = StorageFactory()

            pass

        return StorageFactory._instance

    @staticmethod
    def create(name, *args):
        instance = StorageFactory._singleton()

        if name not in instance._creators.keys():
            raise Exception('{} is invalid '.format(name))
        creator = instance._creators[name]

        if creator is None:
            # return None # 或者 raise 'unsupported object type'
            raise Exception('unsupported object type')
        return creator(*args)

    @staticmethod
    def register(name, creator):
        instance = StorageFactory._singleton()
        instance._creators[name] = creator
