import time
import json
import os, sys
import logging
import traceback
import functools


def log(kind):
    filepath = sys._getframe(1).f_code.co_filename

    def add_log(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            time_cost_logger = logging.getLogger()
            time_cost_logger.setLevel('CRITICAL')  # 设置性能测试的日志级别为 CRITICAL 级别
            # 输出到文件
            chlr = logging.FileHandler(kind, mode='a')
            # 定义日志格式
            fmt = logging.Formatter("%(asctime)s - [%(levelname)s] - %(message)s")

            # 配置默认日志格式
            chlr.setFormatter(fmt)
            # 日志记录器添加handler
            time_cost_logger.addHandler(chlr)
            t1 = time.time()
            res = func(*args, **kwargs)
            t2 = time.time()
            content = {}
            try:

                content['time_cost'] = round(float(t2 - t1), 3)
                content['method'] = func.__name__
                content['file'] = filepath.split(os.sep)[-1]

                content_res = json.dumps(content)
                time_cost_logger.critical(content_res)

            except Exception as e:
                time_cost_logger.warning('%s detail: %s' % (str(e), traceback.format_exc()))
            return res

        return wrapper

    return add_log


def pkg_name_logs(logfile, log_data):

    # 第一步，创建一个logger
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)  # Log等级总开关
    # 第二步，创建一个handler，用于写入日志文件
    rq = time.strftime('%Y%m%d%H%M', time.localtime(time.time()))

    fh = logging.FileHandler(logfile, mode='a')
    fh.setLevel(logging.DEBUG)  # 输出到file的log等级的开关
    # 第三步，定义handler的输出格式
    formatter = logging.Formatter("%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s")
    fh.setFormatter(formatter)
    # 第四步，将logger添加到handler里面
    logger.addHandler(fh)
    # 日志
    logger.info(log_data)

