import os
from osgeo import gdal
from app.utils.create_geojsonl import verify_geotiff, create_default_geojsonl


def write_geojsonl(geojsonl_data: dict, root_path: str):

    for xml_relative_path, properties in geojsonl_data.items():

        xml_path = os.path.join(root_path, xml_relative_path)
        abs_path = os.path.dirname(xml_path)
        filename = os.path.basename(xml_path)
        name = filename.lower().rstrip('.xml').rstrip('.tif.xml').rstrip('.tiff.xml')

        for _file in os.listdir(abs_path):

            tif_path = os.path.join(abs_path, _file)

            if not os.path.isfile(tif_path):
                continue

            _name, _ext = os.path.splitext(_file)

            if _ext.lower() not in ['.tif', '.tiff', ] or _name.lower() != name:
                continue

            break

        # tif = os.path.splitext(xml_relative_path)[0] + '.tif'
        # input_file_path = os.path.join(root_path, tif.strip(r"\\"))

        epsg = verify_geotiff(tif_path, None)

        new_geojsonl = tif_path.split('.')[0] + '.geojsonl'

        # 添加备用字段
        properties["bk1"] = ''
        properties["bk2"] = ''
        properties["bk3"] = ''

        # 增加优先级disp字段和分辨率res字段信息
        ds = gdal.Open(tif_path)
        res = float(ds.GetGeoTransform()[1])
        del ds

        properties['disp'] = 1
        properties['res'] = res

        # 格式化日期
        if len(properties["date"]) < 10:
            properties["date"] = properties["datetime"]

        if '-' in properties["date"]:
            properties["date"] = properties["date"].split(' ')[0]

        if ':' not in properties["datetime"]:
            properties["datetime"] = properties["datetime"] + " 00:00:00"

        create_default_geojsonl(tif_path, new_geojsonl, properties=properties)
